@@ -85,35 +85,36 @@ BOOL CSoundFile::ReadXM(const BYTE *lpStream, DWORD dwMemLength)
 	BYTE pattern_map[256];
 	BOOL samples_used[MAX_SAMPLES];
 	UINT unused_samples;
+	tagXMFILEHEADER xmhead;
 
 	m_nChannels = 0;
 	if ((!lpStream) || (dwMemLength < 0x200)) return FALSE;
 	if (strnicmp((LPCSTR)lpStream, "Extended Module", 15)) return FALSE;
 
 	memcpy(m_szNames[0], lpStream+17, 20);
-	dwHdrSize = bswapLE32(*((DWORD *)(lpStream+60)));
-	norders = bswapLE16(*((WORD *)(lpStream+64)));
+	xmhead = *(tagXMFILEHEADER *)(lpStream+60);
+	dwHdrSize = bswapLE32(xmhead.size);
+	norders = bswapLE16(xmhead.norder);
 	if ((!norders) || (norders > MAX_ORDERS)) return FALSE;
-	restartpos = bswapLE16(*((WORD *)(lpStream+66)));
-	channels = bswapLE16(*((WORD *)(lpStream+68)));
+	restartpos = bswapLE16(xmhead.restartpos);
+	channels = bswapLE16(xmhead.channels);
 	if ((!channels) || (channels > 64)) return FALSE;
 	m_nType = MOD_TYPE_XM;
 	m_nMinPeriod = 27;
 	m_nMaxPeriod = 54784;
 	m_nChannels = channels;
 	if (restartpos < norders) m_nRestartPos = restartpos;
-	patterns = bswapLE16(*((WORD *)(lpStream+70)));
+	patterns = bswapLE16(xmhead.patterns);
 	if (patterns > 256) patterns = 256;
-	instruments = bswapLE16(*((WORD *)(lpStream+72)));
+	instruments = bswapLE16(xmhead.instruments);
 	if (instruments >= MAX_INSTRUMENTS) instruments = MAX_INSTRUMENTS-1;
 	m_nInstruments = instruments;
 	m_nSamples = 0;
-	memcpy(&xmflags, lpStream+74, 2);
-	xmflags = bswapLE16(xmflags);
+	xmflags = bswapLE16(xmhead.flags);
 	if (xmflags & 1) m_dwSongFlags |= SONG_LINEARSLIDES;
 	if (xmflags & 0x1000) m_dwSongFlags |= SONG_EXFILTERRANGE;
-	defspeed = bswapLE16(*((WORD *)(lpStream+76)));
-	deftempo = bswapLE16(*((WORD *)(lpStream+78)));
+	defspeed = bswapLE16(xmhead.speed);
+	deftempo = bswapLE16(xmhead.tempo);
 	if ((deftempo >= 32) && (deftempo < 256)) m_nDefaultTempo = deftempo;
 	if ((defspeed > 0) && (defspeed < 40)) m_nDefaultSpeed = defspeed;
 	memcpy(Order, lpStream+80, norders);
@@ -790,7 +791,7 @@ BOOL CSoundFile::SaveXM(LPCSTR lpszFileName, UINT nPacking)
 			if (nPacking)
 			{
 				if ((!(pins->uFlags & (CHN_16BIT|CHN_STEREO)))
-				 && (CanPackSample(pins->pSample, pins->nLength, nPacking)))
+				 && (CanPackSample((char *)pins->pSample, pins->nLength, nPacking)))
 				{
 					flags[ins] = RS_ADPCM4;
 					xmss.res = 0xAD;
@@ -826,7 +827,7 @@ BOOL CSoundFile::SaveXM(LPCSTR lpszFileName, UINT nPacking)
 			if (pins->pSample)
 			{
 #ifndef NO_PACKING
-				if ((flags[ismpd] == RS_ADPCM4) && (xmih.samples>1)) CanPackSample(pins->pSample, pins->nLength, nPacking);
+				if ((flags[ismpd] == RS_ADPCM4) && (xmih.samples>1)) CanPackSample((char *)pins->pSample, pins->nLength, nPacking);
 #endif // NO_PACKING
 				WriteSample(f, pins, flags[ismpd]);
 			}
